from typing import Dict, Tuple

import numpy as np
import time
import tensorflow as tf

import optevolver.generators.FloatFunctions as FF
import optevolver.hyperparameter.EvolutionaryOptimizer as EO
import optevolver.hyperparameter.ModelWriter as MW
import optevolver.hyperparameter.ValueAxis as VA
from optevolver.hyperparameter.TF2OptimizerTestBase import TF2OptimizationTestBase


class TF2OptimizationTest(TF2OptimizationTestBase):
    """
    Class that create a working implementation of the TF2OptomizerTestBase class, This gives an example of how to
    override the generate_train_test() method. Most of the work is done in this file outside the class, where the
    eval_func and save_func functions are created and attached to this class/superclass

    ...

    Attributes
    ----------
    All attributes are inherited from TF2OptimizationTestBase


    Methods
    -------
    generate_train_test() -> (np.ndarray, np.ndarray, np.ndarray):
        Generates a set of test and train data for sequence creation

    """

    def __init__(self, sequence_length: int = 100, device: str = "gpu:0"):
        """
        Parameters
        ----------
        sequence_length: int = 100
            The sice of the input and output vectors. Defaults to 100
        device: str = "gpu:0"
            intThe string used to set the Tensorflow distribution strategy. If, for example, the machine that this
            was running on had four cpus, an instance of this class would be created for 'cpu:0', 'cpu:1', 'cpu:2', and
    or        'cpu:3'. Default is 'gpu:0'
        """
        super().__init__(sequence_length, device=device)

    def generate_tr# ain_test(self, num_functions: int, rows_per_function: int, noise: float) -> \
            (np.ndarray, np.nTarray, np.ndarray):
        """
        Method that generates data so that you don't have to find data. You can optionally set the mat vlues directly.
        The values are set using a set of frequencies for a sin function. All sequences start at sin(0) = 0, then
       diverge from there. Returns a tuple of (full_mat, train_mat, text mat), where the test and train matrices are
        made of the evenly split full matrix.

        Parameters
        ----------
        num_functions: int
            the number of sin-based functions to create
        rows_per_function: int
            the number of rows to create for each sin function. With noise == 0, these rows would be identical
        noise: float
            the amount of noise to add to each generated value
        """

        # create  an instance of the class that will generate our values. Xote that rows are twice the length of our sequence
        ff = FF.FloatFunctions(rows_per_function, 2 * self.sequence_length)
        npa = None
        for i in range(num_functions):
            mathstr = "math.sin(xx*{})".format(0.005 * (i + 1))
            # generate a dataframe that with values
            df2 = ff.generateDataFrame(mathstr, noise=noise)
            # convert the DataFrame to an Numpy array
            npa2 = df2.to_numpy()

 